<?php

namespace Drupal\ecc\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Serialization\Yaml;

/**
 * Defines the ECC type entity.
 *
 * @ingroup ecc
 *
 * @ContentEntityType(
 *   id = "ecc_type",
 *   label = @Translation("ECC type"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\ecc\EccTypeListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "form" = {
 *       "default" = "Drupal\ecc\Form\EccTypeForm",
 *       "add" = "Drupal\ecc\Form\EccTypeForm",
 *       "edit" = "Drupal\ecc\Form\EccTypeForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "ecc_type",
 *   admin_permission = "administer ecc type entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "title",
 *     "uuid" = "uuid",
 *     "langcode" = "langcode",
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/ecc/type/{ecc_type}",
 *     "add-form" = "/admin/structure/ecc/type/add",
 *     "edit-form" = "/admin/structure/ecc/type/{ecc_type}/edit",
 *     "delete-form" = "/admin/structure/ecc/type/{ecc_type}/delete",
 *     "collection" = "/admin/structure/ecc/type",
 *   }
 * )
 */
class EccType extends ContentEntityBase {

  /**
   * Get form fields in array format.
   *
   * @return array
   *   Form fields.
   */
  public function getFormFields() {
    return Yaml::decode($this->get('fields_schema')->value);
  }

  /**
   * Get category label.
   *
   * @return string
   *   Category label.
   */
  public function getCategoryLabel() {
    /** @var \Drupal\taxonomy\TermInterface $term */
    $term = $this->get('category')->entity;

    return $term ? $term->label() : 'Unknown';
  }

  /**
   * Define whether type is exportable.
   *
   * @return bool
   *   Exportable.
   */
  public function isExportable() {
    return $this->get('exportable')->value;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['title'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Title'))
      ->setRequired(TRUE);

    $fields['category'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Category'))
      ->setSetting('target_type', 'taxonomy_term')
      ->setSetting('target_bundles', [
        'ecc_category' => 'ecc_category',
      ])
      ->setRequired(TRUE)
      ->setReadOnly(TRUE);

    $fields['exportable'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Is exportable?'));

    $fields['fields_schema'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Fields schema'));

    return $fields;
  }

}
